@@ -11,8 +11,10 @@
 #ifndef ADIOS2_BINDINGS_CXX11_CXX11_ENGINE_H_
 #define ADIOS2_BINDINGS_CXX11_CXX11_ENGINE_H_
 
+#include "ADIOSView.h"
 #include "Types.h"
 #include "Variable.h"
+#include "VariableNT.h"
 
 #include "adios2/common/ADIOSMacros.h"
 #include "adios2/common/ADIOSTypes.h"
@@ -88,8 +90,7 @@ public:
      * @param timeoutSeconds
      * @return current step status
      */
-    StepStatus BeginStep(const StepMode mode,
-                         const float timeoutSeconds = -1.f);
+    StepStatus BeginStep(const StepMode mode, const float timeoutSeconds = -1.f);
 
     /**
      * Inspect current logical step
@@ -111,8 +112,7 @@ public:
      * @return span to variable data in engine internal buffer
      */
     template <class T>
-    typename Variable<T>::Span Put(Variable<T> variable, const bool initialize,
-                                   const T &value);
+    typename Variable<T>::Span Put(Variable<T> variable, const bool initialize, const T &value);
 
     /**
      * Put signature that provides access to an internal engine buffer (decided
@@ -139,8 +139,9 @@ public:
      * @exception std::invalid_argument for invalid variable or nullptr data
      */
     template <class T>
-    void Put(Variable<T> variable, const T *data,
-             const Mode launch = Mode::Deferred);
+    void Put(Variable<T> variable, const T *data, const Mode launch = Mode::Deferred);
+
+    void Put(VariableNT &variable, const void *data, const Mode launch = Mode::Deferred);
 
     /**
      * Put data associated with a Variable in the Engine
@@ -158,8 +159,7 @@ public:
      * @exception std::invalid_argument if variable not found or nullptr data
      */
     template <class T>
-    void Put(const std::string &variableName, const T *data,
-             const Mode launch = Mode::Deferred);
+    void Put(const std::string &variableName, const T *data, const Mode launch = Mode::Deferred);
 
     /**
      * Put data associated with a Variable in the Engine
@@ -172,8 +172,12 @@ public:
      * @exception std::invalid_argument if variable is invalid or nullptr &datum
      */
     template <class T>
-    void Put(Variable<T> variable, const T &datum,
-             const Mode launch = Mode::Deferred);
+    void Put(Variable<T> variable, const T &datum, const Mode launch = Mode::Deferred);
+
+#define declare_type(T)                                                                            \
+    void Put(VariableNT &variable, const T &datum, const Mode launch = Mode::Deferred);
+    ADIOS2_FOREACH_STDTYPE_1ARG(declare_type)
+#undef declare_type
 
     /**
      * Put data associated with a Variable in the Engine
@@ -188,8 +192,30 @@ public:
      * @exception std::invalid_argument if variable is invalid or nullptr &datum
      */
     template <class T>
-    void Put(const std::string &variableName, const T &datum,
-             const Mode launch = Mode::Deferred);
+    void Put(const std::string &variableName, const T &datum, const Mode launch = Mode::Deferred);
+
+    /**
+     * The next two Put functions are used to accept a variable, and an
+     * AdiosViews which is a placeholder for Kokkos::View
+     * @param variable contains variable metadata information
+     * @param data represents any user defined object that is not a vector (used
+     * for an AdiosView)
+     * @param launch mode policy, optional for API consistency, internally is
+     * always sync
+     */
+    template <class T, typename U,
+              class = typename std::enable_if<std::is_convertible<U, AdiosView<U>>::value>::type>
+    void Put(Variable<T> variable, U const &data, const Mode launch = Mode::Deferred)
+    {
+        auto bufferView = static_cast<AdiosView<U>>(data);
+#if defined(ADIOS2_HAVE_KOKKOS) || defined(ADIOS2_HAVE_GPU_SUPPORT)
+        auto bufferMem = bufferView.memory_space();
+        auto bufferLayout = bufferView.layout();
+        variable.SetMemorySpace(bufferMem);
+        variable.SetArrayLayout(bufferLayout);
+#endif
+        Put(variable, bufferView.data(), launch);
+    }
 
     /** Perform all Put calls in Deferred mode up to this point.  Specifically,
      * this causes Deferred data to be copied into ADIOS internal buffers as if
@@ -219,6 +245,8 @@ public:
     template <class T>
     void Get(Variable<T> variable, T *data, const Mode launch = Mode::Deferred);
 
+    void Get(VariableNT &variable, void *data, const Mode launch = Mode::Deferred);
+
     /**
      * Get data associated with a Variable from the Engine. Overloaded version
      * to get variable by name.
@@ -239,8 +267,7 @@ public:
      * doesn't exist in IO) or nullptr data
      */
     template <class T>
-    void Get(const std::string &variableName, T *data,
-             const Mode launch = Mode::Deferred);
+    void Get(const std::string &variableName, T *data, const Mode launch = Mode::Deferred);
 
     /**
      * Get single value data associated with a Variable from the Engine
@@ -252,8 +279,17 @@ public:
      * @exception std::invalid_argument if variable is invalid or nullptr &datum
      */
     template <class T>
-    void Get(Variable<T> variable, T &datum,
-             const Mode launch = Mode::Deferred);
+    void Get(Variable<T> variable, T &datum, const Mode launch = Mode::Deferred);
+
+#define declare_type(T)                                                                            \
+    void Get(VariableNT &variable, T &datum, const Mode launch = Mode::Deferred);
+    ADIOS2_FOREACH_STDTYPE_1ARG(declare_type)
+#undef declare_type
+
+#define declare_type(T)                                                                            \
+    void Get(VariableNT &variable, std::vector<T> &datum, const Mode launch = Mode::Deferred);
+    ADIOS2_FOREACH_STDTYPE_1ARG(declare_type)
+#undef declare_type
 
     /**
      * Get single value data associated with a Variable from the Engine
@@ -267,8 +303,7 @@ public:
      * doesn't exist in IO) or nullptr data
      */
     template <class T>
-    void Get(const std::string &variableName, T &datum,
-             const Mode launch = Mode::Deferred);
+    void Get(const std::string &variableName, T &datum, const Mode launch = Mode::Deferred);
 
     /**
      * Get data associated with a Variable from the Engine.
@@ -287,8 +322,7 @@ public:
      * @exception std::invalid_argument for invalid variable
      */
     template <class T>
-    void Get(Variable<T> variable, std::vector<T> &dataV,
-             const Mode launch = Mode::Deferred);
+    void Get(Variable<T> variable, std::vector<T> &dataV, const Mode launch = Mode::Deferred);
 
     /**
      * Get data associated with a Variable from the Engine.
@@ -366,6 +400,29 @@ public:
     template <class T>
     void Get(Variable<T> variable, T **data) const;
 
+    /**
+     * The next two Get functions are used to accept a variable, and an
+     * AdiosViews which is a placeholder for Kokkos::View
+     * @param variable contains variable metadata information
+     * @param data represents any user defined object that is not a vector (used
+     * for an AdiosView)
+     * @param launch mode policy, optional for API consistency, internally is
+     * always sync
+     */
+    template <class T, typename U,
+              class = typename std::enable_if<std::is_convertible<U, AdiosView<U>>::value>::type>
+    void Get(Variable<T> variable, U const &data, const Mode launch = Mode::Deferred)
+    {
+        auto bufferView = static_cast<AdiosView<U>>(data);
+#if defined(ADIOS2_HAVE_KOKKOS) || defined(ADIOS2_HAVE_GPU_SUPPORT)
+        auto bufferMem = bufferView.memory_space();
+        auto bufferLayout = bufferView.layout();
+        variable.SetMemorySpace(bufferMem);
+        variable.SetArrayLayout(bufferLayout);
+#endif
+        Get(variable, bufferView.data(), launch);
+    }
+
     /** Perform all Get calls in Deferred mode up to this point */
     void PerformGets();
 
@@ -376,6 +433,12 @@ public:
      */
     void EndStep();
 
+    /**
+     * Returns True if engine status is between BeginStep()/EndStep() pair,
+     * False otherwise.
+     */
+    bool BetweenStepPairs();
+
     /**
      * Manually flush to underlying transport to guarantee data is moved
      * @param transportIndex
@@ -405,6 +468,9 @@ public:
     std::map<size_t, std::vector<typename Variable<T>::Info>>
     AllStepsBlocksInfo(const Variable<T> variable) const;
 
+    std::map<size_t, std::vector<VariableNT::Info>>
+    AllStepsBlocksInfo(const VariableNT &variable) const;
+
     /**
      * Extracts all available blocks information for a particular
      * variable and step.
@@ -415,8 +481,10 @@ public:
      * found it returns an empty vector
      */
     template <class T>
-    std::vector<typename Variable<T>::Info>
-    BlocksInfo(const Variable<T> variable, const size_t step) const;
+    std::vector<typename Variable<T>::Info> BlocksInfo(const Variable<T> variable,
+                                                       const size_t step) const;
+
+    std::vector<VariableNT::Info> BlocksInfo(const VariableNT &variable, const size_t step) const;
 
     /**
      * Get the absolute steps of a variable in a file. This is for
@@ -458,48 +526,6 @@ private:
     core::Engine *m_Engine = nullptr;
 };
 
-#define declare_template_instantiation(T)                                      \
-                                                                               \
-    extern template typename Variable<T>::Span Engine::Put(                    \
-        Variable<T>, const bool, const T &);                                   \
-    extern template typename Variable<T>::Span Engine::Put(Variable<T>);       \
-    extern template void Engine::Get(Variable<T>, T **) const;
-
-ADIOS2_FOREACH_PRIMITIVE_TYPE_1ARG(declare_template_instantiation)
-#undef declare_template_instantiation
-
-#define declare_template_instantiation(T)                                      \
-    extern template void Engine::Put<T>(Variable<T>, const T *, const Mode);   \
-    extern template void Engine::Put<T>(const std::string &, const T *,        \
-                                        const Mode);                           \
-    extern template void Engine::Put<T>(Variable<T>, const T &, const Mode);   \
-    extern template void Engine::Put<T>(const std::string &, const T &,        \
-                                        const Mode);                           \
-                                                                               \
-    extern template void Engine::Get<T>(Variable<T>, T *, const Mode);         \
-    extern template void Engine::Get<T>(const std::string &, T *, const Mode); \
-    extern template void Engine::Get<T>(Variable<T>, T &, const Mode);         \
-    extern template void Engine::Get<T>(const std::string &, T &, const Mode); \
-                                                                               \
-    extern template void Engine::Get<T>(Variable<T>, std::vector<T> &,         \
-                                        const Mode);                           \
-    extern template void Engine::Get<T>(const std::string &, std::vector<T> &, \
-                                        const Mode);                           \
-                                                                               \
-    extern template void Engine::Get<T>(                                       \
-        Variable<T>, typename Variable<T>::Info & info, const Mode);           \
-    extern template void Engine::Get<T>(                                       \
-        const std::string &, typename Variable<T>::Info &info, const Mode);    \
-                                                                               \
-    extern template std::map<size_t, std::vector<typename Variable<T>::Info>>  \
-    Engine::AllStepsBlocksInfo(const Variable<T> variable) const;              \
-                                                                               \
-    extern template std::vector<typename Variable<T>::Info>                    \
-    Engine::BlocksInfo(const Variable<T> variable, const size_t step) const;
-
-ADIOS2_FOREACH_TYPE_1ARG(declare_template_instantiation)
-#undef declare_template_instantiation
-
 std::string ToString(const Engine &engine);
 
 } // end namespace adios2
