@@ -17,11 +17,12 @@ namespace adios2
 namespace core
 {
 
-std::shared_ptr<Operator> MakeOperator(const std::string &type,
-                                       const Params &parameters);
+std::string OperatorTypeToString(const Operator::OperatorType type);
 
-size_t Decompress(const char *bufferIn, const size_t sizeIn, char *dataOut,
-                  MemorySpace memSpace, std::shared_ptr<Operator> op = nullptr);
+std::shared_ptr<Operator> MakeOperator(const std::string &type, const Params &parameters);
+
+size_t Decompress(const char *bufferIn, const size_t sizeIn, char *dataOut, MemorySpace memSpace,
+                  std::shared_ptr<Operator> op = nullptr);
 
 } // end namespace core
 } // end namespace adios2
